const SpotifyWebApi = require('spotify-web-api-node');
const {homedir} = require("os");
const {existsSync, mkdirSync, writeFileSync, readFileSync} = require("fs");

const configDir = `${homedir()}/.spotify-local`
const spotifyCredentialsFile = `${configDir}/credentials.json`

const ensure = () => {
    if (!existsSync(configDir)) mkdirSync(configDir);
    if (!existsSync(spotifyCredentialsFile)) writeFileSync(spotifyCredentialsFile, '{}');
    return true;
};

SpotifyWebApi.prototype.setExpired = function (expiresIn) {
    const expired = (new Date).getTime() + expiresIn;

    this._setCredential('expired', expired);
}

SpotifyWebApi.prototype.getExpired = function () {
    return this._getCredential('expired') ?? 0;
}

/**
 * @returns {Promise<SpotifyWebApi>}
 */
SpotifyWebApi.prototype.ensure = async function () {
    const timeExpired = this.getExpired();
    const time = (new Date).getTime();
    if (time > timeExpired) {
        const { body } = await this.refreshAccessToken();
        const { access_token, expires_in } = body;
        this.setAccessToken(access_token);
        this.setExpired((new Date).getTime() + expires_in);
        saveSpotifyCredentials();
    }

    return this;
}

const spotify = new SpotifyWebApi();

const saveSpotifyCredentials = () => ensure() && writeFileSync(spotifyCredentialsFile, JSON.stringify(spotify.getCredentials()));
const restoreSpotifyCredentials = () => ensure() && spotify.setCredentials(JSON.parse(readFileSync(spotifyCredentialsFile).toString()));
const hasSpotifyCredentials = () => existsSync(spotifyCredentialsFile);

if (existsSync(spotifyCredentialsFile)) restoreSpotifyCredentials();


module.exports = {
    spotify,
    spotifyCredentialsFile,
    saveSpotifyCredentials,
    restoreSpotifyCredentials,
    hasSpotifyCredentials,
}
